package GraphExplorer;

import com.gif4j.GifEncoder;
import com.gif4j.GifFrame;
import com.gif4j.GifImage;
import java.awt.*;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.image.BufferedImage;
import java.io.*;
import java.net.URL;
import java.util.Arrays;
import javax.swing.JFileChooser;
import javax.swing.JOptionPane;
import javax.swing.filechooser.FileNameExtensionFilter;

public final class Graph extends Canvas {

    private int nodeX[];
    private int nodeY[];
    private int startSelectedIndex = -1;
    private int endSelectedIndex = -1;
    private int centerX = 200;
    private int centerY = 200;
    private int graphWidth = 100;
    private int graphHeight = 100;
    private Edge edges[];
    private int parent[];
    private int numberOfNode = 0;
    private int minCost = 0;
    private long sleepTime = 0;
    private boolean isShowResult = false;
    private final MouseAdapter mouseAdapterDrawEdgeByClick;
    private boolean isStarting = false;
    private boolean isSlow;

    public Graph(int numberOfNode, Edge... e) {
        this();
        this.setEdges(numberOfNode, e);
    }

    Graph() {
        this.mouseAdapterDrawEdgeByClick = new java.awt.event.MouseAdapter() {

            @Override
            public void mouseClicked(MouseEvent e) {
                if (!e.isShiftDown()) {
                    drawEdgeByClick(e);
                }
            }
        };
        this.addMouseListener(new java.awt.event.MouseAdapter() {

            @Override
            public void mouseClicked(MouseEvent e) {
                if (e.isShiftDown()) {
                    moving(e);
                }
            }
        });
    }

    @Override
    public void setSize(int w, int h) {

        super.setSize(w + 20, h + 20);
        this.centerX = w / 5 * 2;
        this.centerY = h / 5 * 2;
        this.graphWidth = w / 4;
        this.graphHeight = h / 4;

        repaint();
    }

    @Override
    public void setSize(Dimension d) {
        this.setSize(d.width, d.height);

    }

    public long getSleepTime() {
        return sleepTime;
    }

    public void setSleepTime(long sleepTime) {
        this.sleepTime = sleepTime;
    }

    public void showSaveDialog() {
        JFileChooser j = new JFileChooser();
        FileNameExtensionFilter graph = new FileNameExtensionFilter("Graph File", "graph");
        FileNameExtensionFilter gif = new FileNameExtensionFilter("Gif Image Animation", "gif");
        j.addChoosableFileFilter(graph);
        j.addChoosableFileFilter(gif);
        j.setAcceptAllFileFilterUsed(false);
        j.setFileFilter(gif);
        j.setSize(300, 200);

        j.showSaveDialog(this);
        if (j.getSelectedFile() == null) {
            return;
        }

        String saveFile = j.getSelectedFile().getPath();
        int dotIndex = saveFile.lastIndexOf(".");
        if (dotIndex == -1) {
            dotIndex = saveFile.length();
        }
        if (j.getFileFilter() == gif) {
            if (!saveFile.toLowerCase().endsWith(".gif")) {
                saveFile = saveFile.substring(0, dotIndex) + ".gif";
            }
            this.saveAnimation(new File(saveFile));

        } else if (j.getFileFilter() == graph) {
            if (!saveFile.toLowerCase().endsWith(".graph")) {
                saveFile = saveFile.substring(0, dotIndex) + ".graph";
            }
            this.saveGraph(new File(saveFile));
        }
    }

    public boolean saveAnimation(File f) {

        GifImage gif = new GifImage();
        gif.setTransparentColor(Color.WHITE);
        if (this.sleepTime < 500) {
            gif.setDefaultDelay(50);
        } else {
            gif.setDefaultDelay((int) this.sleepTime / 10);
        }

        gif.setLoopNumber(0);
        BufferedImage bi = new BufferedImage(this.getWidth(), this.getHeight(), BufferedImage.TYPE_INT_ARGB);
        Graphics2D g = bi.createGraphics();

        for (int i = 0; i < edges.length; i++) {
            g.setColor(Color.BLUE);
            g.drawLine(nodeX[edges[i].start], nodeY[edges[i].start], nodeX[edges[i].end], nodeY[edges[i].end]);
            g.setColor(Color.red);
            g.drawString(edges[i].weight + "", (nodeX[edges[i].start] + nodeX[edges[i].end]) / 2,
                    (nodeY[edges[i].start] + nodeY[edges[i].end]) / 2);
        }
        gif.addGifFrame(new GifFrame(bi));

        for (int i = 0; i < edges.length; i++) {
            g.setColor(new Color(240, 240, 240));
            g.drawLine(nodeX[edges[i].start], nodeY[edges[i].start], nodeX[edges[i].end], nodeY[edges[i].end]);
            g.setColor(new Color(250, 200, 200));
            g.drawString(edges[i].weight + "", (nodeX[edges[i].start] + nodeX[edges[i].end]) / 2,
                    (nodeY[edges[i].start] + nodeY[edges[i].end]) / 2);
        }
        gif.addGifFrame(new GifFrame(bi));
        runKruskal();
        this.isShowResult = false;
        for (int i = 0; i < edges.length; i++) {
            if (edges[i].isResult) {
                g.setColor(Color.BLUE);
                g.drawLine(nodeX[edges[i].start], nodeY[edges[i].start], nodeX[edges[i].end], nodeY[edges[i].end]);
                g.setColor(Color.red);
                g.drawString(edges[i].weight + "", (nodeX[edges[i].start] + nodeX[edges[i].end]) / 2,
                        (nodeY[edges[i].start] + nodeY[edges[i].end]) / 2);
                gif.addGifFrame(new GifFrame(bi));

            }
        }

        Font font = g.getFont();
        g.setFont(new Font(font.getName(), Font.BOLD, 30));
        g.setColor(new Color(150, 150, 230));
        for (int i = 1; i < 5; i++) {
            g.drawString("Min Cost : " + this.getMinCost(), 20 + i, (int) (this.graphHeight * 2.8) + i);
        }
        g.setColor(Color.blue);
        g.drawString("Min Cost : " + this.getMinCost(), 20, (int) (this.graphHeight * 2.8));
        GifFrame gi = new GifFrame(bi);
        gi.setDelay(200);
        gif.addGifFrame(gi);
        try {
            GifEncoder.encode(gif, f);
            return true;
        } catch (Exception ex) {
        }
        return false;
    }

    public void removeAllEdges() {
        this.edges = null;
        this.repaint();
    }

    public void setDrawEdgeByClickEnable(boolean b) {

        if (b) {
            if (!this.isStarting) {
                this.isStarting = true;
                this.addMouseListener(this.mouseAdapterDrawEdgeByClick);
            }
        } else {
            if (this.isStarting) {
                this.removeMouseListener(this.mouseAdapterDrawEdgeByClick);
                this.isStarting = false;
            }
        }
    }

    private void moving(MouseEvent e) {
        if (this.startSelectedIndex < 0) {
            int i = this.getSelectedNodeIndex(e.getX(), e.getY());
            if (i < 0) {
                return;
            }

            this.startSelectedIndex = i;

        } else {
            this.nodeX[startSelectedIndex] = e.getX();
            this.nodeY[startSelectedIndex] = e.getY();
            startSelectedIndex = -1;

        }
        repaint();


    }

    public void reset() {
        this.isShowResult = false;
        if (edges != null) {
            for (int i = 0; i < edges.length; i++) {
                edges[i].setIsResult(false);
            }
        }
        parent = new int[this.numberOfNode];
        for (int i = 0; i < parent.length; i++) {
            parent[i] = -1;
        }
        this.setDrawEdgeByClickEnable(true);
        repaint();
    }

    public void setNumberOfNode() {
        try {
            int n = Integer.parseInt(JOptionPane.showInputDialog("Enter Number Of Node :"));
            if (n > 0) {
                setNumberOfNode(n);
            }
        } catch (HeadlessException | NumberFormatException e) {
        }

    }

    void setSleepTime() {
        try {
            int n = Integer.parseInt(JOptionPane.showInputDialog("Enter Sleep Time (ms):"));
            if (n >= 0) {
                setSleepTime(n);
            }
        } catch (HeadlessException | NumberFormatException e) {
        }
    }

    public void setNumberOfNode(int numberOfNode) {
        if (numberOfNode < this.numberOfNode) {
            this.removeAllEdges();
        }
        this.startSelectedIndex = -1;
        this.endSelectedIndex = -1;
        this.numberOfNode = numberOfNode;
        this.nodeX = new int[numberOfNode];
        this.nodeY = new int[numberOfNode];

        double angle = 0;
        double a = 6.283185307179586 / numberOfNode;
        for (int i = 0; i < numberOfNode; i++) {
            nodeX[i] = (int) (centerX + graphWidth * Math.cos(angle));
            nodeY[i] = (int) (centerY - graphHeight * Math.sin(angle));
            angle += a;
        }
        repaint();
    }

    private void drawEdgeByClick(MouseEvent evt) {

        int i = this.getSelectedNodeIndex(evt.getX(), evt.getY());
        if (i == -1) {
            return;
        }
        if (startSelectedIndex == -1) {
            this.startSelectedIndex = i;
        } else if (endSelectedIndex == -1) {
            this.endSelectedIndex = i;
            if (startSelectedIndex == endSelectedIndex) {
                this.startSelectedIndex = -1;
                this.endSelectedIndex = -1;
                repaint();
                return;
            }
            repaint();
            int wt;
            try {
                wt = Integer.parseInt(JOptionPane.showInputDialog("Enter Weight :"));
                if (wt <= 0) {
                    this.startSelectedIndex = -1;
                    this.endSelectedIndex = -1;
                    return;
                }

                addEdge(new Edge(this.startSelectedIndex + 1, this.endSelectedIndex + 1, wt));
            } catch (HeadlessException | NumberFormatException ex) {
            }
            this.startSelectedIndex = -1;
            this.endSelectedIndex = -1;
        }

        repaint();
        if (this.edges != null) {
            if (Main.requestHelp && this.edges.length == 3) {
                Main.showHelp(3);
            }
        }
    }

    public int getMinCost() {
        return minCost;
    }

    private void drawNode(Graphics g) {
        for (int i = 0; i < numberOfNode; i++) {
            g.setColor(Color.BLACK);
            g.drawString(i + 1 + "", nodeX[i] + 10, nodeY[i] + 10);
            g.fillArc(nodeX[i], nodeY[i], 5, 5, 0, 360);

            if (startSelectedIndex == i) {
                g.setColor(Color.BLUE);
                g.fillArc(nodeX[i] - 5, nodeY[i] - 10, 15, 15, 0, 360);
            } else if (endSelectedIndex == i) {
                g.setColor(Color.BLUE);
                g.fillArc(nodeX[i] - 5, nodeY[i] - 10, 15, 15, 0, 360);
            }
        }
    }

    private void drawResultEdges(Graphics g) {
        for (int i = 0; i < edges.length; i++) {
            g.setColor(new Color(240, 240, 240));
            g.drawLine(nodeX[edges[i].start], nodeY[edges[i].start], nodeX[edges[i].end], nodeY[edges[i].end]);
            g.setColor(new Color(250, 200, 200));
            g.drawString(edges[i].weight + "", (nodeX[edges[i].start] + nodeX[edges[i].end]) / 2,
                    (nodeY[edges[i].start] + nodeY[edges[i].end]) / 2);
        }
        for (int i = 0; i < edges.length; i++) {
            if (edges[i].isResult) {
                if (isSlow) {
                    try {
                        Thread.sleep(this.sleepTime);

                    } catch (Exception ex) {
                    }
                }
                g.setColor(Color.BLUE);
                g.drawLine(nodeX[edges[i].start], nodeY[edges[i].start], nodeX[edges[i].end], nodeY[edges[i].end]);
                g.setColor(Color.red);
                g.drawString(edges[i].weight + "", (nodeX[edges[i].start] + nodeX[edges[i].end]) / 2,
                        (nodeY[edges[i].start] + nodeY[edges[i].end]) / 2);
            }
        }
        Font font = g.getFont();
        g.setFont(new Font(font.getName(), Font.BOLD, 30));
        g.setColor(new Color(150, 150, 230));
        for (int i = 1; i < 5; i++) {
            g.drawString("Min Cost : " + this.getMinCost(), 20 + i, (int) (this.graphHeight * 2.8) + i);
        }
        g.setColor(Color.blue);
        g.drawString("Min Cost : " + this.getMinCost(), 20, (int) (this.graphHeight * 2.8));
        g.setFont(font);
    }

    private void drawAllEdges(Graphics g) {
        for (int i = 0; i < edges.length; i++) {
            g.setColor(Color.GREEN);
            g.drawLine(nodeX[edges[i].start], nodeY[edges[i].start], nodeX[edges[i].end], nodeY[edges[i].end]);
            g.setColor(Color.red);
            g.drawString(edges[i].weight + "", (nodeX[edges[i].start] + nodeX[edges[i].end]) / 2,
                    (nodeY[edges[i].start] + nodeY[edges[i].end]) / 2);
        }
    }

    @Override
    public void paint(Graphics g) {
        g.setColor(Color.WHITE);
        g.fillRect(0, 0, this.graphWidth * 5, this.graphHeight * 5);
        if (edges != null) {
            if (this.isShowResult) {
                this.drawNode(g);
                this.drawResultEdges(g);
            } else {
                this.drawAllEdges(g);
            }
        }
        this.drawNode(g);

        this.isSlow = false;
    }

    public int getSelectedNodeIndex(int x, int y) {
        int t, index = -1, d = 0x7fffffff; //max
        for (int i = 0; i < numberOfNode; i++) {
            int x1 = Math.abs(nodeX[i] - x);
            int y1 = Math.abs(nodeY[i] - y);
            t = (int) Math.sqrt(x1 * x1 + y1 * y1);
            if (t < d && t < 50) {
                d = t;
                index = i;
            }
        }
        return index;
    }

    public void setEdges(int numberOfNode, Edge... e) {
        this.setNumberOfNode(numberOfNode);
        this.setEdges(e);
    }

    public void setEdges(Edge... e) {
        this.edges = e;
        parent = new int[this.numberOfNode];
        for (int i = 0; i < parent.length; i++) {
            parent[i] = -1;
        }
    }

    public void runKruskal() {
        if (edges == null) {
            return;
        }

        Arrays.sort(edges);
        reset();
        this.minCost = 0;
        int count = 1;
        if (edges != null) {
            for (int i = 0; i < this.edges.length; i++) {
                if (checkCycle(edges[i].start, edges[i].end)) {
                    minCost += edges[i].weight;
                    edges[i].setIsResult(true);
                    count++;
                }
                if (count == this.numberOfNode) {
                    break;
                }
            }
        }
        this.isShowResult = true;
        this.isSlow = true;
        repaint();

    }

    public void addEdge(Edge edge) {

        if (edges == null) {
            this.edges = new Edge[0];
        }

        Edge[] newEdges = new Edge[edges.length + 1];
        System.arraycopy(edges, 0, newEdges, 0, edges.length);
        newEdges[edges.length] = edge;
        this.setEdges(newEdges);

    }

    boolean checkCycle(int i, int j) {
        while (parent[i] != -1) {
            i = parent[i];
        }

        while (parent[j] != -1) {
            j = parent[j];
        }

        if (i != j) {
            parent[j] = i;
            return true;
        }
        return false;
    }

    public void readGraph(File f) {
        try (ObjectInputStream o = new ObjectInputStream(new FileInputStream(f))) {
            this.nodeX = (int[]) o.readObject();
            this.nodeY = (int[]) o.readObject();
            this.centerX = (int) o.readObject();
            this.centerY = (int) o.readObject();
            this.graphWidth = (int) o.readObject();
            this.graphHeight = (int) o.readObject();
            this.edges = (Edge[]) o.readObject();
            this.numberOfNode = (int) o.readObject();
        } catch (Exception e) {
        }
        this.repaint();
    }

    public void saveGraph(File f) {
        try {
            try (ObjectOutputStream o = new ObjectOutputStream(new FileOutputStream(f))) {
                o.writeObject(this.nodeX);
                o.writeObject(this.nodeY);
                o.writeObject(this.centerX);
                o.writeObject(this.centerY);
                o.writeObject(this.graphWidth);
                o.writeObject(this.graphHeight);
                o.writeObject(this.edges);
                o.writeObject(this.numberOfNode);

            }
        } catch (Exception ex) {
        }

    }
}
