package GraphExplorer;

import WindowsTools.RegistryTools;
import java.awt.Robot;
import java.awt.Toolkit;
import java.awt.event.*;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.net.URL;
import java.nio.file.Path;
import javax.swing.*;
import javax.swing.filechooser.FileFilter;
import javax.swing.filechooser.FileNameExtensionFilter;

/**
 *
 * @author Ahmad Dehnavi
 */
public class Main {

    static boolean requestHelp, isExistArgment = true;
    private static JButton JBsetSleepTime;
    private static JButton JBhelp;
    private static JButton JBremoveAllEdges;
    private static JButton JBrunKruskal;
    private static JButton JBnewGraph;
    private static JButton JBresetGraph;
    private static JButton JBsaveGraph;
    private static JButton JBabout;

    public static void main(String[] args) {
        if (args.length == 0) {
            setRegistryKeys();
            isExistArgment = false;
        }

        final JFrame jf = new JFrame("Graph Explorer ");
        jf.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
        final Graph g = new Graph();
        copyIcon();
        g.setDrawEdgeByClickEnable(true);
        JMenuBar jm = new JMenuBar();
        JBrunKruskal = new JButton("Run Kruskal");

        JBrunKruskal.addActionListener(new ActionListener() {

            @Override
            public void actionPerformed(ActionEvent e) {
                g.runKruskal();
                g.setDrawEdgeByClickEnable(false);

                showHelp(5);
            }
        });
        JBnewGraph = new JButton("New Graph");
        JBnewGraph.addActionListener(new ActionListener() {

            @Override
            public void actionPerformed(ActionEvent e) {

                g.setNumberOfNode();
                showHelp(2);

            }
        });
        JBresetGraph = new JButton("Reset Graph");
        JBresetGraph.addActionListener(new ActionListener() {

            @Override
            public void actionPerformed(ActionEvent e) {
                g.reset();
                showHelp(6);
            }
        });

        JBremoveAllEdges = new JButton("Remove All Edges");
        JBremoveAllEdges.addActionListener(new ActionListener() {

            @Override
            public void actionPerformed(ActionEvent e) {
                g.removeAllEdges();
                showHelp(8);
            }
        });
        JBsetSleepTime = new JButton("Set Sleep Time");
        JBsetSleepTime.addActionListener(new ActionListener() {

            @Override
            public void actionPerformed(ActionEvent e) {
                g.setSleepTime();
                showHelp(4);
            }
        });
        JBsaveGraph = new JButton("Save");
        JBsaveGraph.addActionListener(new ActionListener() {

            @Override
            public void actionPerformed(ActionEvent e) {
                g.showSaveDialog();

                showHelp(7);
            }
        });
        JBhelp = new JButton("Help");
        JBhelp.addActionListener(new ActionListener() {

            @Override
            public void actionPerformed(ActionEvent e) {
                requestHelp = true;
                jf.setLocation(0, 0);
                jf.setSize(Toolkit.getDefaultToolkit().getScreenSize());
                showHelp(1);
                g.removeAllEdges();
            }
        });

        JBabout = new JButton("About");
        JBabout.addActionListener(new ActionListener() {

            @Override
            public void actionPerformed(ActionEvent e) {
                new About().setVisible(true);
            }
        });

        jm.add(JBnewGraph);
        jm.add(JBsetSleepTime);
        jm.add(JBrunKruskal);
        jm.add(JBresetGraph);
        jm.add(JBsaveGraph);
        jm.add(JBremoveAllEdges);
        jm.add(JBhelp);
        jm.add(JBabout);
        jf.setJMenuBar(jm);
        jf.addComponentListener(new ComponentAdapter() {

            @Override
            public void componentResized(ComponentEvent e) {
                g.setSize(jf.getSize());
            }
        });
        jf.add(g);
        g.setSize(400, 400);
        jf.pack();
        jf.setLocationRelativeTo(null);
        try {
            UIManager.setLookAndFeel("com.sun.java.swing.plaf.nimbus.NimbusLookAndFeel");
            SwingUtilities.updateComponentTreeUI(jf);
        } catch (Exception ex) {
        }

        jf.setVisible(true);
        JLabel jl = new JLabel("Open Graph.");
        FileDrop fileDrop = new FileDrop(g, new FileDrop.Listener() {

            @Override
            public void filesDropped(java.io.File[] files) {
                g.readGraph(files[0]);
            }
        });

        jf.addWindowListener(new WindowAdapter() {

            @Override
            public void windowClosing(WindowEvent e) {
                g.saveGraph(new File(System.getProperty("user.home") + "\\Graph Explorer\\last.graph"));
                super.windowClosing(e);
            }

            @Override
            public void windowOpened(WindowEvent e) {
                super.windowOpened(e);
                if (!isExistArgment) {
                    g.readGraph(new File(System.getProperty("user.home") + "\\Graph Explorer\\last.graph"));
                }
            }
        });
        if (isExistArgment) {
            g.readGraph(new File(args[0]));
        }
    }

    private static void setRegistryKeys() {
        try {

            File f = new File("t");
            String dir = f.getAbsolutePath();
            dir = dir.substring(0, dir.length() - 1);
            RegistryTools.associcateFileExtention(System.getProperty("user.home") + "\\Graph Explorer\\graph icon.ico", "Graph Explorer File",
                    new File("java -jar \"" + dir + "\\GraphExplorer.jar\""), "graph");
        } catch (Exception ex) {
            ex.printStackTrace();
        }

    }

    static void showHelp(int i) {
        int x = 1;
        if (requestHelp) {
            if (helpCounter == i) {
                switch (i) {
                    case 1://new graph

                        x = JOptionPane.showConfirmDialog(null, "1) Click 'New Graph' Button.& Enter 3", "Step 1", JOptionPane.YES_NO_OPTION);
                        helpCounter = 2;
                        break;
                    case 2://draw edges & move it
                        x = JOptionPane.showConfirmDialog(null, "2) Draw Edges By Click on Two Node.\nYou Can Move Node By Press 'Shift' + Click on Node'", "Step 2", JOptionPane.YES_NO_OPTION);
                        helpCounter = 3;
                        break;
                    case 3://set time

                        x = JOptionPane.showConfirmDialog(null, "Click 'set Sleep time' Button", "step 3", JOptionPane.YES_NO_OPTION);
                        helpCounter = 4;
                        JBsetSleepTime.grabFocus();
                        break;
                    case 4://run kruskal

                        x = JOptionPane.showConfirmDialog(null, "Click 'Run Kruskal' Button", "Step 4", JOptionPane.YES_NO_OPTION);
                        helpCounter = 5;
                        JBrunKruskal.grabFocus();
                        break;
                    case 5://reset

                        x = JOptionPane.showConfirmDialog(null, "Click 'Reset Graph' Button", "Step 5", JOptionPane.YES_NO_OPTION);
                        helpCounter = 6;
                        JBresetGraph.grabFocus();
                        break;
                    case 6://save graph & gif

                        x = JOptionPane.showConfirmDialog(null, "Click 'Save' Button", "Step 6", JOptionPane.YES_NO_OPTION);
                        helpCounter = 7;
                        JBsaveGraph.grabFocus();
                        break;
                    case 7://remove edges

                        x = JOptionPane.showConfirmDialog(null, "Click 'Remove All Edges' Button", "Step 7", JOptionPane.YES_NO_OPTION);
                        helpCounter = 8;
                        JBremoveAllEdges.grabFocus();
                        break;
                    case 8://final

                        JOptionPane.showMessageDialog(null, "Tutorial End");
                        helpCounter = 1;
                        JBnewGraph.grabFocus();
                        break;
                }
            }
            if (x == 1) {
                requestHelp = false;
                helpCounter = 1;
            }

        }


    }

    public static void copyIcon() {
        try {
            File f = new File(System.getProperty("user.home") + "\\Graph Explorer");
            if (!f.exists()) {
                f.mkdirs();
            }
            File f2 = new File(System.getProperty("user.home") + "\\Graph Explorer\\graph icon.ico");
            if (f2.exists()) {
                return;
            }

            URL url = ClassLoader.getSystemResource("/img/graphicon.ico");

            if (url == null) {
                File iconfile;
                File file = new File("graph icon.ico");
                if (file.exists()) {
                    iconfile = file;
                } else {
                    JOptionPane.showMessageDialog(null, "Can Not Copy Graph Icon to " + System.getProperty("user.home")
                            + "\\Graph Explorer"
                            + "\n please Copy Icon 'Graph Icon.ico' to this Folder.");
                    JFileChooser j = new JFileChooser();
                    j.setDialogTitle("Select Icon File.");
                    j.setFileFilter(new FileNameExtensionFilter("Icon File", "ico"));
                    j.showOpenDialog(null);
                    iconfile = j.getSelectedFile();
                }
                FileOutputStream out;
                try (FileInputStream in = new FileInputStream(iconfile)) {
                    out = new FileOutputStream(System.getProperty("user.home") + "\\Graph Explorer\\graph icon.ico");
                    byte[] b = new byte[1024];
                    int Bytes;
                    while ((Bytes = in.read(b)) != -1) {
                        out.write(b, 0, Bytes);
                    }
                }
                out.close();
            } else {
                FileOutputStream out;
                try (FileInputStream in = new FileInputStream(new File(url.toURI()))) {
                    out = new FileOutputStream(System.getProperty("user.home") + "\\Graph Explorer\\graph icon.ico");
                    byte[] b = new byte[1024];
                    int Bytes;
                    while ((Bytes = in.read(b)) != -1) {
                        out.write(b, 0, Bytes);
                    }
                }
                out.close();
            }
        } catch (Exception ex) {
            ex.printStackTrace();
        }
    }
    private static int helpCounter = 1;
}
